/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef HELPERS_HPP__
#define HELPERS_HPP__

#include <libelf.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <functional>
#include <string>
#include <unistd.h>

#pragma pack(push, 1)

struct Notefile32 {
	uint32_t start;
	uint32_t end;
	uint32_t file_ofs;
};

struct Notefile64 {
	uint64_t start;
	uint64_t end;
	uint64_t file_ofs;
};

struct ProgramTableHeader32 {
	uint32_t p_type;
	Elf32_Off p_offset;
	Elf32_Addr p_vaddr;
	Elf32_Addr p_paddr;
	uint32_t p_filesz;
	uint32_t p_memsz;
	uint32_t p_flags;
	uint32_t p_align;
};

struct ProgramTableHeader64 {
	uint32_t p_type;
	uint32_t p_flags;
	Elf64_Off p_offset;
	Elf64_Addr p_vaddr;
	Elf64_Addr p_paddr;
	uint64_t p_filesz;
	uint64_t p_memsz;
	uint64_t p_align;
};

struct SymData {
	unsigned long start;
	Elf *elf;
	Elf64_Shdr shdr;
	Elf_Data *data;
	int fd;
	int count;
	struct SymData *next;
};

#pragma pack(pop)

struct Elf32 {
	typedef Elf32_Ehdr Ehdr;
	typedef Elf32_Phdr Phdr;
	typedef Elf32_Addr Addr;
	typedef Elf32_Shdr Shdr;
	typedef Elf32_Off Off;
	typedef Elf32_Section Section;
	typedef Elf32_Versym Versym;
	typedef Elf32_Half Half;
	typedef Elf32_Sword Sword;
	typedef Elf32_Word Word;
	typedef Elf32_Sxword Sxword;
	typedef Elf32_Xword Xword;
	typedef ProgramTableHeader32 ProgramTableHeader;
	typedef Notefile32 Notefile;
	typedef Elf32_auxv_t auxv_t;
	typedef Elf32_Dyn Dyn;
	typedef uint32_t uint;
	static const char ELFCLASS = ELFCLASS32;
};

struct Elf64 {
	typedef Elf64_Ehdr Ehdr;
	typedef Elf64_Phdr Phdr;
	typedef Elf64_Addr Addr;
	typedef Elf64_Shdr Shdr;
	typedef Elf64_Off Off;
	typedef Elf64_Section Section;
	typedef Elf64_Versym Versym;
	typedef Elf64_Half Half;
	typedef Elf64_Sword Sword;
	typedef Elf64_Word Word;
	typedef Elf64_Sxword Sxword;
	typedef Elf64_Xword Xword;
	typedef ProgramTableHeader64 ProgramTableHeader;
	typedef Notefile64 Notefile;
	typedef Elf64_auxv_t auxv_t;
	typedef Elf64_Dyn Dyn;
	typedef uint64_t uint;
	static const char ELFCLASS = ELFCLASS64;
};

class Guardian {
 private:
	const std::function<void()> m_start;
	const std::function<void()> m_end;
 public:
	Guardian(const std::function<void()> &start, const std::function<void()> &end)
								: m_start(start), m_end(end) { m_start(); }
	~Guardian() { m_end(); }
};

bool fileExists(const std::string &path) {
	struct stat buf;
	return stat(path.c_str(), &buf) == 0;
}

#endif  // HELPERS_HPP__
