/*
 * crash-manager
 * Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __DEF_UTIL_H__
#define __DEF_UTIL_H__

#include <stdbool.h>
#include <sys/types.h>

#define ARRAY_SIZE(name) (sizeof(name)/sizeof(name[0]))

#ifndef __CONSTRUCTOR__
#define __CONSTRUCTOR__ __attribute__ ((constructor))
#endif

#define fprintf_fd(fd, fmt, ...) dprintf(fd, fmt, ##__VA_ARGS__)

#ifdef __cplusplus
extern "C" {
#endif

int write_fd(int fd, const void *buf, int len);

int copy_bytes(int destfd, int srcfd, off_t *ncopied);

int copy_file(char *dst, char *src);

int move_file(char *dst, char *src);

int dump_file_write_fd(int dfd, char *src);

int fsync_path(char *const path);

int make_dir(const char *path, const char *name, int mode);

int remove_dir(const char *path, int del_dir);

int get_exec_pid(const char *execpath);

int get_file_count(char *path);

off_t get_directory_usage(char *path);

int log_kmsg(char *fmt, ...);

int find_crash_tid(int pid);

bool get_exe_path(pid_t pid, char *outbuf, size_t outbuf_len);

typedef bool (*charp0filter)(char *buf, int size, int datalen);
bool filter_drop_trailing_whitespace(char *buf, int size, int datalen);
bool read_proc_file(pid_t pid, const char * const file, char *outbuf, size_t bufsize, charp0filter filter);

char* concatenate(char *const vec[]);

bool string_ends_with(const char *string, const char *suffix);

bool file_exists(const char *path);

bool file_exists_in_dir(const char *dir_path, const char *file_name);

bool write_to_file(const char *content, const char *base_dir, const char *file_name);

#ifdef __cplusplus
}
#endif

/**
 * @}
 */
#endif
